@@ -2,9 +2,10 @@
 import logging
 from itertools import zip_longest
 from pathlib import Path
-from typing import List, Optional, Type, TypeVar
+from typing import List, Optional, Set, Type, TypeVar
 
 from alembic.autogenerate import comparators
+from alembic.autogenerate.api import AutogenContext
 from flupy import flu
 from sqlalchemy.orm import Session
 from sqlalchemy.sql.elements import TextClause
@@ -42,6 +43,15 @@ class ReplaceableEntity:
         self.signature: str = coerce_to_unquoted(normalize_whitespace(signature))
         self.definition: str = escape_colon_for_sql(strip_terminating_semicolon(definition))
 
+    @property
+    def type_(self) -> str:
+        """In order to support calls to `run_name_filters` and
+        `run_object_filters` on the AutogenContext object, each
+        entity needs to have a named type.
+        https://alembic.sqlalchemy.org/en/latest/api/autogenerate.html#alembic.autogenerate.api.AutogenContext.run_name_filters
+        """
+        raise NotImplementedError()
+
     @classmethod
     def from_sql(cls: Type[T], sql: str) -> T:
         """Create an instance from a SQL string"""
@@ -173,6 +183,14 @@ def register_entities(
     **Parameters:**
 
     * **entities** - *List[ReplaceableEntity]*: A list of entities (PGFunction, PGView, etc) to monitor for revisions
+
+    **Deprecated Parameters:**
+
+    .. deprecated:: 0.5.1 for removal in 0.6.0
+
+    *Configure schema and object inclusion/exclusion with `include_name` and `include_object` in `env.py`. For more information see https://alembic.sqlalchemy.org/en/latest/autogenerate.html#controlling-what-to-be-autogenerated*
+
+
     * **schemas** - *Optional[List[str]]*: A list of SQL schema names to monitor. Note, schemas referenced in registered entities are automatically monitored.
     * **exclude_schemas** - *Optional[List[str]]*: A list of SQL schemas to ignore. Note, explicitly registered entities will still be monitored.
     * **entity_types** - *Optional[List[str]]*: A list of ReplaceableEntity classes to consider during migrations. Other entity types are ignored
@@ -184,7 +202,7 @@ def register_entities(
 
     @comparators.dispatch_for("schema")
     def compare_registered_entities(
-        autogen_context,
+        autogen_context: AutogenContext,
         upgrade_ops,
         sqla_schemas: Optional[List[Optional[str]]],
     ):
@@ -197,25 +215,29 @@ def register_entities(
                     f"PGFunction with identity {ident} was registered multiple times"
                 )
 
-        all_schema_references: List[str] = []
+        # https://alembic.sqlalchemy.org/en/latest/autogenerate.html#controlling-what-to-be-autogenerated
+        # if EnvironmentContext.configure.include_schemas is True, all non-default "scehmas" should be included
+        # pulled from the inspector
+        include_schemas: bool = autogen_context.opts["include_schemas"]
 
-        # User registered schemas + automatically registered schemas (from SQLA Metadata)
-        if schemas is not None:
-            for schema in schemas:
-                if schema is not None:
-                    all_schema_references.append(schema)
-
-        if sqla_schemas is not None:
-            to_add = [x for x in sqla_schemas if x]
-            all_schema_references.extend(to_add)
-
-        for entity in entities:
-            all_schema_references.append(entity.schema)
+        reflected_schemas = set(
+            autogen_context.inspector.get_schema_names() if include_schemas else []  # type: ignore
+        )
+        sqla_schemas: Set[Optional[str]] = sqla_schemas or {}  # type: ignore
+        manual_schemas = set(schemas or [])  # Deprecated for remove in 0.6.0
+        entity_schemas = {x.schema for x in entities}  # from ReplaceableEntity instances
+        all_schema_references = reflected_schemas | sqla_schemas | manual_schemas | entity_schemas  # type: ignore
 
         # Remove excluded schemas
-        observed_schemas = [
-            x for x in set(all_schema_references) if x not in (exclude_schemas or [])
-        ]
+        observed_schemas: Set[str] = {
+            schema_name
+            for schema_name in all_schema_references
+            if (
+                schema_name
+                not in (exclude_schemas or [])  # user defined. Deprecated for remove in 0.6.0
+                and schema_name not in {"information_schema", None}
+            )
+        }
 
         # Solve resolution order
         try:
@@ -243,6 +265,14 @@ def register_entities(
             if entity.__class__ not in allowed_entity_types:
                 continue
 
+            if not include_entity(entity, autogen_context, reflected=False):
+                logger.debug(
+                    "Ignoring local entity %s %s due to AutogenContext filters",
+                    entity.__class__.__name__,
+                    entity.identity,
+                )
+                continue
+
             try:
                 transaction = connection.begin()
                 sess = Session(bind=connection)
@@ -267,7 +297,7 @@ def register_entities(
                         entity.identity,
                     )
                 else:
-                    logger.info(
+                    logger.debug(
                         "Detected NoOp op for %s %s",
                         entity.__class__.__name__,
                         entity.identity,
@@ -299,11 +329,22 @@ def register_entities(
 
                     # Check for functions that were deleted locally
                     for db_entity in db_entities:
+
+                        if not include_entity(db_entity, autogen_context, reflected=True):
+                            logger.debug(
+                                "Ignoring local entity %s %s due to AutogenContext filters",
+                                entity.__class__.__name__,
+                                entity.identity,
+                            )
+                            continue
+
                         for local_entity in local_entities:
                             if db_entity.identity == local_entity.identity:
                                 break
                         else:
                             # No match was found locally
+                            # If the entity passes the filters,
+                            # we should create a DropOp
                             upgrade_ops.ops.append(DropOp(db_entity))
                             logger.info(
                                 "Detected DropOp op for %s %s",
@@ -313,3 +354,37 @@ def register_entities(
 
         finally:
             transaction.rollback()
+
+
+def include_entity(entity: T, autogen_context: AutogenContext, reflected: bool) -> bool:
+    """The functions on the AutogenContext object
+    are described here:
+    https://alembic.sqlalchemy.org/en/latest/api/autogenerate.html#alembic.autogenerate.api.AutogenContext.run_name_filters
+
+    The meaning of the function parameters are explained in the corresponding
+    definitions in the EnvironmentContext object:
+    https://alembic.sqlalchemy.org/en/latest/api/runtime.html#alembic.runtime.environment.EnvironmentContext.configure.params.include_name
+
+    This will only have an impact for projects which set include_object and/or include_name in the configuration
+    of their Alembic env.
+    """
+    name = f"{entity.schema}.{entity.signature}"
+    parent_names = {
+        "schema_name": entity.schema,
+        # At the time of writing, the implementation of `run_name_filters` in Alembic assumes that every type of object
+        # will either be a table or have a table_name in its `parent_names` dict. This is true for columns and indexes,
+        # but not true for the type of objects supported in this library such as views as functions. Nevertheless, to avoid
+        # a KeyError when calling `run_name_filters`, we have to set some value.
+        "table_name": f"Not applicable for type {entity.type_}",
+    }
+    # According to the Alembic docs, the name filter is only relevant for reflected objects
+    if reflected:
+        name_result = autogen_context.run_name_filters(name, entity.type_, parent_names)
+    else:
+        name_result = True
+
+    # Object filters should be applied to object from local metadata and to reflected objects
+    object_result = autogen_context.run_object_filters(
+        entity, name, entity.type_, reflected=reflected, compare_to=None
+    )
+    return name_result and object_result
